\

marker =lcd-graphics=
: _wrtData csl [ $22 ] literal wrtCmd wrtData ;

\ Draw a pixel with the foreground color
\ This function assumes it will be nested within hwAccess functions
: _pixel		( x y --- )
    over over 		( x y --- x y x y)
    setBounds		( x y x y --- )
    fgColor @ _wrtData
;

\ Draw a pixel with the foreground color
\ This function obtains hwAccess
: pixel			( x y --- )
    \ Obtain exclusive access to all hardware I/O    
    hwAccess[
        _pixel
    \ Release exclusive access to all hardware I/O    
    ]hwAccess
    clearBounds
;

variable y2
variable x2 
variable y1
variable x1

\ Draw a rectangle filled with foreground color
\ This function assumes it will be nested within hwAccess functions
: _filledRect		( x1 y1 x2 y2 --- )
    y2 ! x2 ! y1 ! x1 !
    x1 @ y1 @ x2 @ y2 @ setBounds
    fgColor @ _wrtData
    x2 @ x1 @ - abs 1+
    y2 @ y1 @ - abs 1+
    [ 0 ] literal do dup
       [ 0 ] literal do 
        wrl 
        wrh 
        loop
    loop 
drop
;

\ Draw a rectangle filled with foreground color
\ This function obtains hwAccess
: filledRect		( x1 y1 x2 y2 --- )
    \ Obtain exclusive access to all hardware I/O    
    hwAccess[
        _filledRect
    \ Release exclusive access to all hardware I/O    
    ]hwAccess
;

\ Draw horizontal line with foreground color
\ This function assumes it will be nested within hwAccess functions
: _hLine		( x y len --- )
    >r			( x y len --- x y )	
    over over		( x y --- x y x y )
    swap		( x y x y --- x y y x )
    r>			( x y y x --- x y y x len )
    +			( x y y x len --- x y y x+len )
    swap		( x y y x+len --- x y x+len y )
    _filledRect
;

\ Draw horizontal line with foreground color
\ This function obtains hwAccess
: hLine			( x y len --- )
    hwAccess[
        _hLine
    ]hwAccess
;

\ Draw vertical line with foreground color
\ This function assumes it will be nested within hwAccess functions
: _vLine		( x y len --- )
    2>r			( x y len --- x )
    dup			( x --- x x )
    2r>			( x x --- x x y len )
    over		( x x y len --- x x y len y )
    +			( x x y len y --- x x y y+len )
    rot			( x x y y+len --- x y y+len x )
    swap		( x y y+len x --- x y x y+len )
    _filledRect
;

\ Draw vertical line with foreground color
\ This function obtains hwAccess
: vLine			( x y len --- )
    hwAccess[
        _vLine
    ]hwAccess
;

variable y2
variable x2 
variable y1
variable x1

\ Draw a rectangle with foreground color
: rect			( x1 y1 x2 y2 --- )
    y2 ! x2 ! y1 ! x1 !
    hwAccess[
        x1 @ y1 @ x2 @ x1 @ - abs _hLine
        x1 @ y2 @ x2 @ x1 @ - abs _hLine
        x1 @ y1 @ y2 @ y1 @ - abs _vLine
        x2 @ y1 @ y2 @ y1 @ - abs _vLine
    ]hwAccess
;

variable radius
variable y
variable x

\ Draw filled circle with foreground color
: filledCircle		( x y radius --- ) 
    radius ! y ! x !
    1 radius @ negate
    hwAccess[
        do
            [ 1 ] literal radius @ negate
            do
                i i * j j * + radius @ radius @ * <
                if x @ i + y @ j + i negate 2* _hLine
                   x @ i + y @ j - i negate 2* _hLine
                   leave
                then
            loop
        loop
    ]hwAccess
;

variable dx
variable dy
variable error

\ Draw a circle with foreground color
: circle		( x y radius --- )
    radius ! y ! x !
    radius @ 0 <>
    if
        0 dx !
        radius @ dy !
        3 radius @ 2 * - error !
        hwAccess[
            begin
            dy @ dx @ >
            while
                x @ dx @ - y @ dy @ - _pixel
                x @ dy @ - y @ dx @ - _pixel
                x @ dy @ + y @ dx @ - _pixel
                x @ dx @ + y @ dy @ - _pixel
                x @ dx @ - y @ dy @ + _pixel
                x @ dy @ - y @ dx @ + _pixel
                x @ dy @ + y @ dx @ + _pixel
                x @ dx @ + y @ dy @ + _pixel
                [ 0  ] literal error @ >
                if
                    error @ dx @ [ 4 ] literal * [ 6 ] literal + + error !
                else
                    error @ dx @ dy @ - [ 4 ] literal  * [ 10 ] literal  + + error !
                    dy @ 1- dy !
                then
                dx @ 1+ dx !
            repeat
        ]hwAccess
    then
;


\ Swap the value of two variables
: swapVars		( v1addr v2addr --- )
    over over 		( v1addr v2addr --- v1addr v2addr v1addr v2addr )
    @                   ( v1addr v2addr v1addr v2addr --- v1addr v2addr v1addr v2 )
    swap @		( v1addr v2addr v1addr v2 --- v1addr v2addr v2 v1 )
    >r			( v1addr v2addr v2 v1 --- v1addr v2addr v2 )
    rot			( v1addr v2addr v2 --- v2addr v2 v1addr )
    !			( v2addr v2 v1addr --- v2addr )
    r>			( v2addr --- v2addr v1 ) 
    swap		( v2addr v1 --- v1 v2addr ) 
    !			( v1 v2addr --- )
;

variable steep
variable derror

\ Draw line with foreground color
\ This function assumes it will be nested within hwAccess functions
: _line			( x1 y1 x2 y2 --- )
    y2 ! x2 ! y1 ! x1 !
    y2 @ y1 @ - abs x2 @ x1 @ - abs > steep !
    steep @
    if
        x1 y1 swapVars
        x2 y2 swapVars
    then
    x1 @ x2 @ >
    if
        x1 x2 swapVars
        y1 y2 swapVars
    then
    y2 @ y1 @ - abs derror !
    y1 @ y2 @ >
    if
        -1 dy !
    else
         1 dy !
    then
    x2 @ x1 @ - dx !
    dx @ 1 rshift error !
    y1 @ y !
    x2 @ 1+ x1 @ 
    do
        steep @
        if
            y @ i _pixel
        else
            i y @ _pixel
        then
        error @ derror @ - error !
        error @ 0<
        if
            dy @ y +!
            dx @ error +!
        then
    loop
;

\ Draw line with foreground color
\ This function obtains hwAccess
: line			( x1 y1 x2 y2 --- )
    hwAccess[
        _line
    ]hwAccess
;


\ Storage for graphics cursor
variable cursorX
variable cursorY

\ Setup defaults
0 cursorX !
0 cursorY !

: moveCursor		( x y --- )
  cursorY !
  cursorX !
;

\ Draw a line to the given location from the position of the graphics cursor
\ then update the cursor's position with the new location
: lineTo		( x y --- )
    2>r
    cursorX @ cursorY @ 2r> 
    over over cursorY ! cursorX !
    line
;

