\ On screen printing functions for the SSD1289 65K Color LCD Controller
\ Written for AmForth
\ By: Craig A. Lindley
\ Last Update: 11/13/2103
\ Must have ssd1289Graphics.frt, utilities.frt, font.frt  loaded

marker _PRINT_

\ Return cell at offset into font specified on the stack
: font@				( offset --- cell ) 
    FONT + @i 
;

\ Render a row of font data
: renderFontRow			( font_row_data --- )
   [ 0 ] literal [ FWIDTH 1- ] literal			\ For all bits of font row data 
    do
        dup			( font_row_data --- font_row_data font_row_data )
        \ Create bitmask for individual bit
        [ 1 ] literal i lshift and
        if			\ If bit set
            fgColor @ wrtData
        else		\ If bit not set
            bgColor @ wrtData
        then
        [ -1 ] literal
    +loop
    drop
;

\ Print a character from the font
\ This function assumes it will be nested within hwAccess functions
: _printChar			( c --- )
csl [ $22 ] literal wrtCmd
    \ Calculate start of char data in font
    [ $20 ] literal - CELLSPERCHAR *	( c --- offset )
    
    \ Set bounds around char
    cursorX @ cursorY @ cursorX @ [ FWIDTH 1- ] literal
 + cursorY @ [ FHEIGHT 1- ] literal + setBounds

    [ CELLSPERCHAR ] literal [ 0 ] literal		\ For all rows of font char data
    do
        dup			( offset --- offset offset )
	i +			( offset offset --- offset offset+i ) 
        font@			( offset offset+i --- offset font_row_data )
        [ ROWSPERCELL ] literal [ 1 ] literal =		\ One row per cell ?
        if			\ Yes
            renderFontRow
	else			\ No
            dup 		( offset font_row_data --- offset font_row_data font_row_data )
            [ 8 ] literal rshift renderFontRow
            [ $FF ] literal and  renderFontRow
	then
    loop drop
;

: setXY
[ FHEIGHT ] literal * cursorY !
[ FWIDTH ] literal * cursorX !
;

\ Print a character from the font
\ This function obtains hwAccess
: printChar			( c x y --- )
    hwAccess[
	setXY
        _printChar
    ]hwAccess
;

: lcd_emit ( c -- )
_printChar
[ FWIDTH ] literal cursorX +!
;

  \ Switch emit to LCD
: emit->lcd ( -- )
    ['] lcd_emit ['] emit defer! ;

  \ Switch emit to serial
: emit->tx0 ( -- )
\    ['] tx-isr ['] emit defer!
    ['] tx-poll ['] emit defer!  
;

  \ Number to LCD
: lcd_. ( n -- )
    emit->lcd . emit->tx0 ;

  \ String in RAM to LCD
: lcd_type ( adr u -- )
    emit->lcd type  emit->tx0 ;

\ ~ilya
\ Печатаем из флэшки
: lcd_typef  dup 2/ swap over 2* - + over + swap
?do i @i dup 
[ $ff ] literal  and lcd_emit >< 
[ $ff ] literal and ?dup if lcd_emit then loop ;

variable addr
variable count
variable count2
variable flag

\ Print string from flash horizontally
: horizPrintString		( addr count --- )

    count ! addr !
    count @ 1 rshift count2 !
    count @ 2 mod 0 <> flag !

    count2 @ 0
    do
        addr @ @i dup
        [ $FF ] literal and _printChar
        [ FWIDTH ] literal cursorX +!
        [ 8 ] literal rshift _printChar 
        [ FWIDTH ] literal cursorX +!
         [ 1 ] literal addr +!
    loop
    flag @
    if
        addr @ @i [ $FF ] literal and _printChar
        FWIDTH cursorX +! 
    then
;

\ Print string from flash vertically
: vertPrintString		( addr count --- )

    count ! addr !
    count @ 1 rshift count2 !
    count @ 2 mod 0 <> flag !

    count2 @ 0
    do
        addr @ @i dup 
        $FF and _printChar
        FHEIGHT cursorY +!
        8 rshift _printChar
        FHEIGHT cursorY +!
        1 addr +!
    loop
    flag @
    if
        addr @ @i $FF and _printChar
        FHEIGHT cursorY +! 
    then
;

variable dps

\ Compile a string to flash and return its address and count
: f" 
    $22 parse 		( addr count --- )
    count ! addr !
    count @ 1 rshift count2 !
    count @ 2 mod 0 <> flag !

    dp dps !
    count2 @ 0
    do
        addr @ c@ 1 addr +!
        addr @ c@ 8 lshift or 1 addr +!
        ,
    loop
    flag @
    if
        addr @ c@ ,
    then
    dps @ count @
;
 
\ Define a label that places itself on the display
: label:			( x y addr count --- )
    create
    , , , ,
    does>
        dup over over
        3 + @i swap 2 + @i moveCursor
        1+ @i swap @i
;



